module Boggler
  class Cell
    attr_reader :row, :column, :letter, :word, :previous

    MOVES = [
      [-1, -1],
      [-1,  0],
      [-1,  1],
      [ 0, -1],
      [ 0,  1],
      [ 1, -1],
      [ 1,  0],
      [ 1,  1],
    ]

    def initialize(
        grid, row, column, dictionary = nil, used = nil, previous = nil)
      @row    = row
      @column = column
      @letter = grid[row - 1][column - 1]
      @grid   = grid
      @previous = previous
      @dictionary = dictionary || Dictionary.words

      set_used used
      @used[row][column] = true

      @word = ((previous && previous.word) || '') + @letter
      set_dictionary
    end

    def next_cell
      begin
        iterator.next
      rescue StopIteration
        nil
      end
    end

    def valid_word?
      !!@dictionary['']
    end

    def possible_word?
      !!@dictionary
    end

    private

    def set_dictionary
      case word.length
      when 1, 2
        @dictionary = {}
      when 3
        @dictionary = Dictionary.words[@word]
      else
        @dictionary = @dictionary[@word[-1]]
      end
    end

    def set_used(used)
      @used = used && used.dup
      @used ||= {}

      @grid.size.times do |i|
        row = used && used[i + 1] && used[i + 1].dup
        @used[i + 1] = row || {}
      end
    end

    def iterator
      @iterator ||= neighbors.each
    end

    def neighbors
      return @neighbors if @neighbors

      @neighbors = []

      MOVES.each do |move|
        next_row    = row + move.first
        next_column = column + move.last

        if next_row > 0 && next_row <= @grid.size &&
            next_column > 0 && next_column <= @grid.size &&
            !@used[next_row][next_column]
          @neighbors << Cell.new(
            @grid, next_row, next_column, @dictionary, @used, self)
        end
      end

      @neighbors
    end
  end
end
